# Analyis performed based on the tutorial available at https://www.zoology.ubc.ca/prog/diversitree/doc/diversitree-tutorial.pdf

# Packages
library(diversitree)
library (ape)

# Loading tree with frog species sampled in our study
phy <- read.tree("tree_pyron_112_species")

# Loading the frog distribtuion in the coastal and inland regions. Zero indicates species occuring in both regions.
areas <- read.csv("areas.csv", header = T, row.names = 1, sep = ",")

# Checking if species names are identical
setdiff(rownames(areas), phy$tip.label) 
setdiff(phy$tip.label, rownames(areas)) 

# Creating vector
areas_AB <- areas$Area 
names(areas_AB)<- rownames(areas) 

# make a starting pass for the GeoSSE analysis
p <- starting.point.geosse(phy)

#############################################################################################################
# Model construction and constraining
#############################################################################################################
# Constructing full model

full.lik <- make.geosse(phy, areas_AB)

# Creating constrained models
lik1 <- constrain(full.lik, sAB ~ 0) # constrain the speciation between A&B regions to 0
lik2 <- constrain(full.lik, sA ~ sB) # constrain speciation between regions as equal
lik3 <- constrain(full.lik, xA ~ xB) # constrain extinction of AtoB and BtoA as equal.
lik4 <- constrain(full.lik, dA ~ dB) # constrain dispersal of AtoB and BtoA as equal.
lik5 <- constrain(full.lik, dA ~ dB, xA ~ xB) # constrain dispersal directional dispersal as equal
lik6 <- constrain(full.lik, dA ~ dB, sA ~ sB) # constrain d
lik7 <- constrain(full.lik, xA ~ xB, sA ~ sB) # equal diversification
lik8 <- constrain(full.lik, dA ~ dB, xA ~ xB, sA ~ sB) # constrain dispersal directional dispersal as equal


m.full <- find.mle(full.lik, p) #lnLik = -594.08 #this estimates the marginal likelihood of our model
(p <- coef(m.full)) #pull the coefficient of this model to apply it to alternatives
ml1 <- find.mle(lik1, p[argnames(lik1)]) #lnLik = -603.69 # it's worse
ml2 <- find.mle(lik2, p[argnames(lik2)]) #lnLik = -595.35 #as above (here lik2), also worse
ml3 <- find.mle(lik3, p[argnames(lik3)]) #lnLik = -594.08 #as above (here lik3), better
ml4 <- find.mle(lik4, p[argnames(lik4)]) #lnLik = -594.97 #as above (here lik3), also worse
ml5 <- find.mle(lik5, p[argnames(lik5)]) #lnLik = -606.24 #as above (here lik3), also worse
ml6 <- find.mle(lik6, p[argnames(lik6)]) #lnLik = -594.99 #as above (here lik3), also worse
ml7 <- find.mle(lik7, p[argnames(lik7)]) #lnLik = -608.10 #as above (here lik3), also worse
ml8 <- find.mle(lik8, p[argnames(lik8)]) #lnLik = -608.10 #as above (here lik3), also worse


############################################################################################################
# Maximum likelihood
############################################################################################################
# ML parameter estimation and model comparisons:
round(rbind(full = coef(m.full),
            no.sAB = coef(ml1, TRUE),
            eq.spe = coef(ml2, TRUE),
            eq.ext = coef(ml3,TRUE),
            eq.disp = coef(ml4, TRUE),
            eq.disp_ext_SAB = coef(ml5, TRUE),
            eq.disp_ext = coef(ml6, TRUE),
            eq.disp_ext_esp = coef(ml7, TRUE),
            eq.ext_SAB = coef(ml8, TRUE)
            ),4) 

# The anova above will give us the ability to determine if we can confidently reject alternative models/scenarios
library(bbmle)
AICtab(m.full, no.sAB = ml1, eq.div = ml2, eq.ext = ml3, eq.disp = ml4, 
      m5 = ml5, m6 = ml6, n7 = ml7, n8 = ml8, base = TRUE,  weights = TRUE,
      delta = TRUE, sort = TRUE, logLik = T) 

#################################################################################################
# Markov chain Monte Carlo
#################################################################################################

# ML rate estimates as a starting point. 
p <- coef(ml5) 
prior <- make.prior.exponential(1/2) #give our parameters large, exponential priors


# Use a pilot run to obtain reasonable step sizes:
set.seed(1) #set seed for reproducibility, which shouldn't be an issue if your sampling is large enough
tmp <- mcmc(lik5, p, nsteps=1000, prior=prior, w=1, print.every=0) # do a test run to determine the appropriate range 
# for the steps (w)
w <- diff(sapply(tmp[2:6], quantile, c(0.025, 0.975)))

# Now the real analysis, which will take awhile to run:

mcmc2 <- mcmc(lik5, p, nsteps = 1000, prior = prior, w = w)


col2 <- c("green", "blue")
mcmc2diff <- with(mcmc2, data.frame(div.A = sA-xB, div.B = sB-xB))
profiles.plot(mcmc2diff, col.line=col2, xlab="", ylab="")
legend("topright", c("sA", "sB"), col=col1, lty=1)

